package jp.sourceforge.acerola3d.a3;

import javax.vecmath.*;

import java.awt.*;

class A3Balloon implements Component2D {
    A3Object a3;
    String description = "none";
    double rightOffsetX;
    double rightOffsetY;
    double leftOffsetX;
    double leftOffsetY;
    double topOffsetX;
    double topOffsetY;
    double bottomOffsetX;
    double bottomOffsetY;
    int width;
    int height;
    A3Object.BalloonDir direction = A3Object.BalloonDir.RIGHT;

    A3Balloon(String s,A3Object a3) {
        description = s;
        this.a3 = a3;
    }
    public void paint(Graphics2D g,A3CanvasInterface canvas) {
        Vector3d v = a3.getLoc();
        if (!a3.lockedA3)
            v = canvas.virtualCSToPhysicalCS(v);
        Vector3d vv = null;
        if (direction==A3Object.BalloonDir.RIGHT) {
            vv = new Vector3d(rightOffsetX,rightOffsetY,0.0);
        } else if (direction==A3Object.BalloonDir.LEFT) {
            vv = new Vector3d(leftOffsetX,leftOffsetY,0.0);
        } else if (direction==A3Object.BalloonDir.TOP) {
            vv = new Vector3d(topOffsetX,topOffsetY,0.0);
        } else if (direction==A3Object.BalloonDir.BOTTOM) {
            vv = new Vector3d(bottomOffsetX,bottomOffsetY,0.0);
        }
        vv.scale(a3.getScale());
        if (!a3.lockedA3)
            vv.scale(1.0/canvas.getCameraScale());
        Vector3d p3d = new Vector3d(v.x+vv.x,v.y+vv.y,v.z+vv.z);
        Point p = canvas.physicalCSToCanvas(new Point3d(p3d));

        int i0 = description.length();
        int i1 = i0/10;
        int i2 = i0%10;
        String desc[] = new String[i1+(i2==0?0:1)];
        for (int i=0;i<desc.length;i++) {
            int j0 = i*10;
            int j1 = i*10+10;
            j1 = j1>description.length()?description.length():j1;
            desc[i]=description.substring(j0,j1);
        }
        FontMetrics fm = g.getFontMetrics();
        width = fm.stringWidth("ww")*10+60;
        height = fm.getHeight()*desc.length+60;
        width  = width<50 ?50 :width;
        height = height<50?50:height;

        int x[] = {+0,+10,+10};
        int y[] = {+0,+ 5, -5};
        if (direction==A3Object.BalloonDir.RIGHT) {
            ;
        } else if (direction==A3Object.BalloonDir.LEFT) {
            for (int i=0;i<x.length;i++)x[i]=-x[i];
        } else if (direction==A3Object.BalloonDir.TOP) {
            for (int i=0;i<x.length;i++){int tmp = x[i];x[i]=y[i];y[i]=tmp;}
            for (int i=0;i<y.length;i++)y[i]=-y[i];
        } else if (direction==A3Object.BalloonDir.BOTTOM) {
            for (int i=0;i<x.length;i++){int tmp = x[i];x[i]=y[i];y[i]=tmp;}
        }
        for (int i=0;i<x.length;i++){x[i]=x[i]+p.x;y[i]=y[i]+p.y;}
        g.setColor(Color.white);
        g.fillPolygon(x,y,3);
        if (direction==A3Object.BalloonDir.RIGHT) {
            ;
        } else if (direction==A3Object.BalloonDir.LEFT) {
            p.x = p.x - width + 40;
        } else if (direction==A3Object.BalloonDir.TOP) {
            p.x = p.x - width/2 + 20;
            p.y = p.y - height/2 - 10;
        } else if (direction==A3Object.BalloonDir.BOTTOM) {
            p.x = p.x - width/2 + 20;
            p.y = p.y + height/2 + 10;
        }

        g.fillRect(p.x+      10,p.y-height/2+10,width-60,height-20);
        g.fillRect(p.x+      20,p.y-height/2+ 0,width-80,height   );
        g.fillOval(p.x+      10,p.y-height/2+ 0,      20,       20);
        g.fillOval(p.x+width-70,p.y-height/2+ 0,      20,       20);
        g.fillOval(p.x+      10,p.y+height/2-20,      20,       20);
        g.fillOval(p.x+width-70,p.y+height/2-20,      20,       20);

        g.setColor(Color.black);
        for (int i=0;i<desc.length;i++) {
            g.drawString(desc[i],p.x+30,p.y+i*fm.getHeight()-height/2+20);
        }
    }
    public void setString(String s) {
        description = s;
    }
    public void setDir(A3Object.BalloonDir d) {
        direction = d;
    }
    public void setOffset(A3Object.BalloonDir d,double x,double y) {
        direction = d;
        if (d==A3Object.BalloonDir.RIGHT) {
            rightOffsetX = x;
            rightOffsetY = y;
        } else if (d==A3Object.BalloonDir.LEFT) {
            leftOffsetX = x;
            leftOffsetY = y;
        } else if (d==A3Object.BalloonDir.TOP) {
            topOffsetX = x;
            topOffsetY = y;
        } else if (d==A3Object.BalloonDir.BOTTOM) {
            bottomOffsetX = x;
            bottomOffsetY = y;
        }
    }
}
